@@ -1,8 +1,9 @@
-# Copyright 2013 by Zheng Ruan (zruan1991@gmail.com).
-# All rights reserved.
-# This code is part of the Biopython distribution and governed by its
-# license.  Please see the LICENSE file that should have been included
-# as part of this package.
+# Copyright 2013 by Zheng Ruan (zruan1991@gmail.com). All rights reserved.
+#
+# This file is part of the Biopython distribution and governed by your
+# choice of the "Biopython License Agreement" or the "BSD 3-Clause License".
+# Please see the LICENSE file that should have been included as part of this
+# package.
 """Code for dealing with Codon Alignment.
 
 CodonAlignment class is inherited from MultipleSeqAlignment class. This is
@@ -10,15 +11,16 @@ the core class to deal with codon alignment in biopython.
 """
 
 import warnings
+from math import erfc
+from math import sqrt
 
-from Bio.Align import MultipleSeqAlignment
-from Bio.SeqRecord import SeqRecord
-from Bio.Data import CodonTable
 from Bio import BiopythonWarning
-
-
-from Bio.codonalign.codonseq import _get_codon_list, CodonSeq, cal_dn_ds
-from Bio.codonalign.chisq import chisqprob
+from Bio.Align import MultipleSeqAlignment
+from Bio.codonalign.codonseq import _get_codon_list
+from Bio.codonalign.codonseq import cal_dn_ds
+from Bio.codonalign.codonseq import CodonSeq
+from Bio.Data import CodonTable
+from Bio.SeqRecord import SeqRecord
 
 
 class CodonAlignment(MultipleSeqAlignment):
@@ -65,7 +67,7 @@ class CodonAlignment(MultipleSeqAlignment):
         rows = len(self._records)
         lines = [
             "CodonAlignment with %i rows and %i columns (%i codons)"
-            % (rows, self.get_alignment_length(), self.get_aln_length(),)
+            % (rows, self.get_alignment_length(), self.get_aln_length())
         ]
 
         if rows <= 60:
@@ -300,6 +302,8 @@ def _get_codon2codon_matrix(codon_table):
     Elements in the matrix are number of synonymous and nonsynonymous
     substitutions required for the substitution.
     """
+    import copy
+
     base_tuple = ("A", "T", "C", "G")
     codons = [
         i
@@ -307,7 +311,7 @@ def _get_codon2codon_matrix(codon_table):
         if "U" not in i
     ]
     # set up codon_dict considering stop codons
-    codon_dict = codon_table.forward_table
+    codon_dict = copy.deepcopy(codon_table.forward_table)
     for stop in codon_table.stop_codons:
         codon_dict[stop] = "stop"
     # count site
@@ -398,7 +402,7 @@ def _dijkstra(graph, start, end):
     node = end
     distance = 0
     # While we are not arrived at the beginning
-    while not (node == start):
+    while node != start:
         if path.count(node) == 0:
             path.insert(0, node)  # Insert the predecessor of the current node
             node = P[node]  # The current node becomes its predecessor
@@ -430,9 +434,11 @@ def _prim(G):
     Code is adapted from
     http://programmingpraxis.com/2010/04/09/minimum-spanning-tree-prims-algorithm/
     """
-    from math import floor
     from collections import defaultdict
-    from heapq import heapify, heappop, heappush
+    from heapq import heapify
+    from heapq import heappop
+    from heapq import heappush
+    from math import floor
 
     nodes = []
     edges = []
@@ -487,7 +493,6 @@ def _G_test(site_counts):
     #   Apply continuity correction for Chi-square test.
     from math import log
 
-    # from scipy.stats import chi2
     G = 0
     tot = sum(site_counts)
     tot_syn = site_counts[0] + site_counts[2]
@@ -502,9 +507,9 @@ def _G_test(site_counts):
     ]
     for obs, ex in zip(site_counts, exp):
         G += obs * log(obs / ex)
-    G *= 2
-    # return 1-chi2.cdf(G, 1) # only 1 dof for 2x2 table
-    return chisqprob(G, 1)
+    # with only 1 degree of freedom for a 2x2 table,
+    # the cumulative chi-square distribution reduces to a simple form:
+    return erfc(sqrt(G))
 
 
 if __name__ == "__main__":
